local lastWeaponEnt = null;
local playerInTaunt = false;

AddTimer(-1, -10, function()
{
    local inCond = self.InCond(TF_COND_TAUNTING);
    if (inCond && !playerInTaunt)
    {
        playerInTaunt = true;
    }
    else if (!inCond && playerInTaunt)
    {
        playerInTaunt = false;
        FixViewmodels();
    }

    local weaponEnt = self.GetActiveWeapon();

    if (weaponEnt == lastWeaponEnt)
        return;
    lastWeaponEnt = weaponEnt;
    if (!weaponEnt)
        return;

    local weaponScript = weaponEnt.GetScriptScope();
    if ("OnSwitch" in weaponScript)
        weaponScript.OnSwitch(self, weaponScript.weaponTT, weaponEnt);
});

function FixViewmodels()
{
    local weaponEnt, weaponTT;
	for (local i = 0; i < 7; i++)
        if ((weaponEnt = GetPropEntityArray(self, "m_hMyWeapons", i))
            && (weaponTT = safeget(weaponEnt.GetScriptScope(), "weaponTT", null)))
                FixViewmodel(weaponEnt, weaponTT)
}

function FixViewmodel(weaponEnt, weaponTT)
{
    if ("viewmodel" in weaponTT)
    {
        local viewmodelIndex = GetModelIndex(weaponTT.viewmodel);

        weaponEnt.SetModel(weaponTT.viewmodel);
        weaponEnt.SetCustomViewModelModelIndex(viewmodelIndex);
        SetPropInt(weaponEnt, "m_nModelIndex", viewmodelIndex);
        SetPropInt(weaponEnt, "m_iViewModelIndex", viewmodelIndex);
    }
    if ("worldmodel" in weaponTT)
    {
        SetPropBool(weaponEnt, "m_bBeingRepurposedForTaunt", true);
        PrecacheModel(weaponTT.worldmodel);
        ChangeWorldModel(weaponEnt, weaponTT.worldmodel);
    }
}

function CreateWeapon(weaponTT)
{
    local weaponEnt = Entities.CreateByClassname(weaponTT.baseclass);
    SetPropInt(weaponEnt, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", weaponTT.baseid);
    SetPropBool(weaponEnt, "m_AttributeManager.m_Item.m_bInitialized", true);
    SetPropBool(weaponEnt, "m_bValidatedAttachedEntity", true);
	SetPropBool(weaponEnt, "m_bClientSideAnimation", true);
    weaponEnt.SetTeam(self.GetTeam());
    weaponEnt.SetOwner(self);
	SetPropEntity(weaponEnt, "m_hOwnerEntity", self);
    SetPropBool(weaponEnt, "m_bForcePurgeFixedupStrings", true);
    Entities.DispatchSpawn(weaponEnt);
    return weaponEnt;
}

function EquipWeapon(weaponTT)
{
    local weaponEnt = CreateWeapon(weaponTT);
    self.Weapon_Equip(weaponEnt);

    weaponEnt.ValidateScriptScope();
    weaponEnt.GetScriptScope().weaponTT <- weaponTT;

    FixViewmodel(weaponEnt, weaponTT);

    if ("OnEquip" in weaponTT)
        weaponTT.OnEquip(self, weaponTT, weaponEnt);
    if ("OnSwitch" in weaponTT)
        weaponEnt.GetScriptScope().OnSwitch <- weaponTT.OnSwitch;
    if ("OnFire" in weaponTT)
        IncludeScript(projectDir + "/_charlib/weapon_events.nut", weaponEnt.GetScriptScope())

    return weaponEnt;
}