antiWinLoop <- false;
::hasRoundEnded <- false;
roundWinner <- worldspawn;

function EndRound(winner = null)
{
    if (hasRoundEnded)
        return;
    hasRoundEnded <- true;
    if (!winner)
        winner = PlayerInstanceFromIndex(GetFirstPlace()[0]);
    if (!winner)
        winner = worldspawn;
    roundWinner = winner;

    local roundWin = SpawnEntityFromTable("game_round_win",
    {
        win_reason = "0",
        force_map_reset = "1", //not having
        TeamNum = "0",         //these 3 lines
        switch_teams = "0"     //causes the crash when trying to fire game_round_win
    });
    SetPropBool(roundWin, "m_bForcePurgeFixedupStrings", true);
    EntFireByHandle(roundWin, "SetTeam", ""+winner.GetTeam(), 0.1, null, null);
    EntFireByHandle(roundWin, "RoundWin", "", 0.1, null, null);
    return;
}

OnGameEvent("teamplay_win_panel_post", 0, function(params)
{
    SendGlobalGameEvent("tf_game_over", {});

    EmitSoundEx({
        sound_name = "player/taunt_bell.wav",
        volume = 1,
        sound_level = 150,
        filter_type = RECIPIENT_FILTER_GLOBAL
    });

    foreach(player in GetPlayers())
    {
        if (player == roundWinner)
            continue;
        player.AddCustomAttribute("no_attack", 1, 25);
        player.RemoveCond(TF_COND_CRITBOOSTED);
        foreach (ent in player.GetAttachments())
            if (IsValid(ent) && startswith(ent.GetClassname(), "tf_weapon"))
                KillIfValid(ent);
        player.Weapon_Switch(player.GetScriptScope().EquipWeapon(GetWeaponTypeByName("loser")));
        player.GetScriptScope().RefreshCharacter(true);
    }

    RunWithDelay(4, StartVictoryScreen);
});

::GetSortedScores <- function()
{
    local array = [];
    foreach(k, v in GetScores())
        array.push([k, v]);
    return array.sort(function(it, that) {
        return that[1] - it[1];
    });
}

::GetLeaders <- function()
{
    local scores = GetSortedScores();
    if (scores.len() <= 0)
        return [];
    local maxScore = scores[0][1];
    local result = [];
    foreach(entry in scores)
    {
        if (entry[1] == maxScore)
            result.push(entry[0]);
        else
            break;
    }
    return result;
}

//===========================================================
//Post-Round Win Screen
//===========================================================

::onVictoryScreen <- false;

function StartVictoryScreen()
{
    DoEntFire("logic_victory_screen", "Trigger", "", -1, null, null);

    local cameraSpawn = Entities.FindByName(null, "victory_screen_camera");

    local camera = SpawnEntityFromTable("point_viewcontrol", {
        acceleration = 15,
        angles = cameraSpawn.GetAbsAngles(),
        origin = cameraSpawn.GetOrigin(),
        deceleration = 15,
        speed = 15,
        spawnflags = 28,
        target = "camera_target",
        vscripts = "ffa/gamemode/camera_fix.nut"
    });

    RunWithDelay(1, function() {
        ::onVictoryScreen <- true;
        foreach(player in GetClients())
        {
            player.SetHudHideFlags(HIDEHUD_ALL);
            player.SetScriptOverlayMaterial("");
            EntFireByHandle(env_screenoverlay, "StopOverlays", "", -1, player, player);
            player.RemoveCond(TF_COND_TAUNTING);
            EntFireByHandle(camera, "Enable", "", -1, player, player);
        }
    });
    AddTimer(-1, function(parent) {
        if (!IsValid(parent))
            return;
        local frame = GetPropInt(self, "m_ubInterpolationFrame");
        self.SetOrigin(parent.GetOrigin());
        SetPropInt(self, "m_ubInterpolationFrame", frame);
    }, Entities.FindByName(null, "victory_screen_train"), camera)

    local scores = GetSortedScores();
    local players = [
        PlayerInstanceFromIndex(scores[0][0]),
        PlayerInstanceFromIndex(scores[1][0]),
        PlayerInstanceFromIndex(scores[2][0])
    ]
    local merc_models = [
        Entities.FindByName(null, "victory_mercmodel_first"),
        Entities.FindByName(null, "victory_mercmodel_second"),
        Entities.FindByName(null, "victory_mercmodel_third")
    ]
    local texts = [
        Entities.FindByName(null, "victory_text_first"),
        Entities.FindByName(null, "victory_text_second"),
        Entities.FindByName(null, "victory_text_third")
    ]
    local subtexts = [
        Entities.FindByName(null, "victory_subtext_first"),
        Entities.FindByName(null, "victory_subtext_second"),
        Entities.FindByName(null, "victory_subtext_third")
    ]

    for (local i = 0; i < 3; i++)
    {
        if (!players[i] || !merc_models[i])
        {
            KillIfValid(merc_models[i]);
            EntFireByHandle(texts[i], "SetText", "", -1, null, null);
            EntFireByHandle(subtexts[i], "SetText", "", -1, null, null);
            continue;
        }
        local playerScript = players[i].GetScriptScope();

        SetEntityColor(merc_models[i], playerScript.CalcMercColor());
        EntFireByHandle(texts[i], "SetText", GetPropString(players[i], "m_szNetname"), -1, null, null);
        EntFireByHandle(subtexts[i], "SetText", format("Score: %d", scores[i][1]), -1, null, null);
        SetEntityColor(merc_models[i], playerScript.CalcMercColor());

        playerScript.DressUpVictoryMerc(merc_models[i]);
    }
}

{
    local cameras = [];
    for (local camera = null; camera = Entities.FindByClassname(camera, "point_viewcontrol");)
    {
        cameras.push(camera);
        EntFireByHandle(camera, "Kill", "", 3, null, null);
    }
    foreach(player in GetClients())
    {
        player.SetHudHideFlags(0);
        SetPropInt(player, "m_lifeState", 0);
        SetPropInt(player, "m_takedamage", 2);
        foreach (camera in cameras)
            EntFireByHandle(camera, "Disable", "", -1, player, player);
    }
}