self.SetHudHideFlags(/*HIDEHUD_PIPES_AND_CHARGE | HIDEHUD_METAL |*/ HIDEHUD_BUILDING_STATUS | HIDEHUD_CLOAK_AND_FEIGN);

if (redTeamOnly && self.GetTeam() != TF_TEAM_RED)
{
    self.SwitchTeam(TF_TEAM_RED);
    self.ForceRespawn();
}

local spawnInvulShield = null;
local spawnInvulRunoutTS = Time() + 3;
local moveKeys = IN_ATTACK | IN_ATTACK2 | IN_FORWARD | IN_BACK | IN_MOVELEFT | IN_MOVERIGHT;

OnSelfEvent("init_custom_character", 0, function()
{
    CreateViewModel(characterSettings.viewmodel);

    SetPropInt(self, "m_Shared.m_bShieldEquipped", 0);
    AssembleArsenal(false);

    //Add Merc's attributes
    self.AddCustomAttribute("cancel falling damage", 1, -1);
    self.AddCustomAttribute("crit mod disabled hidden", 0, -1);
    self.AddCustomAttribute("no double jump", 1, -1);
    self.AddCustomAttribute("rocket jump damage reduction", 0.4, -1);
    self.AddCond(TF_COND_BURNING_PYRO);

    //On-spawn invul

    if (GetRoundState() == 3)
        self.AddCustomAttribute("no_attack", 1, 5.2);
    else if (!RandomInt(0, 3))
        BuildSound("respawn").Delay(1.5).Voice().SinglePlayer().Play();

    //Spawn sound
    if (GetRoundState() >= 4)
        BuildSound("respawn_sfx").Play();

    spawnInvulRunoutTS = Time() + (GetRoundState() == 3 ? 5 : 2);

    spawnInvulShield = SpawnEntityFromTable("prop_dynamic", {
        model = "models/effects/resist_shield/resist_shield.mdl",
        disableshadows = 1,
        origin = self.GetCenter(),
        angles = self.EyeAngles(),
    })
    SetPropBool(spawnInvulShield, "m_bForcePurgeFixedupStrings", true);
    EntFireByHandle(spawnInvulShield, "Kill", "", GetRoundState() == 3 ? 5 : 2, null, null);
    AddTimer(-1, function(player)
    {
        if (!IsValidPlayer(player) || ((GetPropInt(player, "m_nButtons") & moveKeys) && GetRoundState() > 3))
        {
            self.Kill();
            return;
        }

        local frame = GetPropInt(self, "m_ubInterpolationFrame");
        self.SetOrigin(player.GetOrigin());
        SetPropInt(self, "m_ubInterpolationFrame", frame);

    }, self, spawnInvulShield);
});

function RefreshCharacter(keepCurrentHP = true)
{
    local oldHP = self.GetHealth();
    local weaponTT = GetWeaponTypeFromEntity(self.GetActiveWeapon());
    local playerClass = safeget(weaponTT, "playerclass", null);

    if (playerClass)
        self.SetPlayerClass(playerClass);
    else
        playerClass = self.GetPlayerClass();
    self.AddCustomAttribute("move speed bonus",
        1.04 / MERC_MOVESPEED_MULTIPLIER[playerClass],
        -1);

    FixViewmodels();

    local model = safeget(weaponTT, "playermodel", null);
    if (model)
        self.SetCustomModelWithClassAnimations(model);
    SetEntityColor(self, CalcMercColor());

    self.SetHealth(keepCurrentHP ? oldHP : 150);
    self.SetMaxHealth(150);
    self.RemoveCustomAttribute("max health additive penalty");
    self.AddCustomAttribute("max health additive penalty", MERC_HP_OFFSETS[playerClass], -1);
}

OnSelfEvent("player_death", -100, function(params)
{
    local attacker = GetPlayerFromParams(params, "attacker");
    if (!IsValidPlayer(attacker) || GetScore(attacker) < 24)
        self.SetCustomModelWithClassAnimations("models/gungame/player/mercenary_invis.mdl");
    self.SetScriptOverlayMaterial("");
    game_text_hop_bar.Display(" ", self);
    if (!RandomInt(0, 2))
        BuildSound("pain_critical_death").Voice().SoundLevel(90).Play();
});

OnSelfEvent("OnTakeDamage", function(params)
{
    if (Time() < spawnInvulRunoutTS)
    {
        params.damage = 0;
        params.early_out = true;
        return 117115;
    }
});

OnSelfEvent("OnDealDamage", -100, function(params)
{
    if ((params.damage_type & 128) && self.InCond(TF_COND_BLASTJUMPING))
        params.damage *= 2;
});